from .interactive import interact #needs to be imported first, because it sets the matplotlib backend
from .plot import show_load_displacement_curve, show_animation, show_deformation_plot
from .plot import Plot2D, Animation3D, DeformationPlot3D
